import {render} from 'preact'
import {App} from './app.tsx'
import './index.css'
import * as proto from './proto'
import {example} from './proto'
import WasmWorker from './worker.js?worker'

const Command = proto.example.Command

render(<App />, document.getElementById('app') as HTMLElement)

import State = example.FocusChange.State;
import Fibonacci = example.Fibonacci;
import * as localforage from "localforage";

const worker = new WasmWorker()
worker.onmessage = async (event: MessageEvent) => {
    if (event.data.message) { // Log message
        console.log(event.data.message)
    } else { // Protobuf
        const buffer = new Uint8Array(event.data.payload)

        switch (event.data.contains) {
            case "command":
                const command = Command.decode(buffer)

                switch (command.type) {
                    case example.Command.Type.LOG:
                        if (command.version == 0) {
                            console.log("Message has invalid version: ", event.data)
                        } else {
                            console.log("Message incoming: ", command)
                        }
                        break
                    case example.Command.Type.READY:
                        console.log("WASM reports ready")
                        const stored = await localforage.getItem('fibonacciSeq')
                        if (stored) {
                            console.log("store retrieval successful")
                            console.log(stored)
                            worker.postMessage({
                                contains: "fibonacci",
                                payload: stored,
                            })
                        }
                        const innerMsg = proto.example.FocusChange.create({
                            version: 1,
                            state: State.FOCUSED
                        })
                        worker.postMessage({
                            contains: "focus_change",
                            payload: proto.example.FocusChange.encode(innerMsg).finish(),
                        })
                }
                break
            case "fibonacci":
                const fibonacci = Fibonacci.decode(buffer)
                console.log(fibonacci.secondToLast, fibonacci.last)
                localforage.setItem('fibonacciSeq', buffer)
        }

    }
}

window.addEventListener('focus', () => {
    const msg = proto.example.FocusChange.create({
        version: 1,
        state: State.FOCUSED
    })
    worker.postMessage({
        contains: "focus_change",
        payload: proto.example.FocusChange.encode(msg).finish(),
    })
    console.log("sent", msg)
})

window.addEventListener('blur', () => {
    const msg = proto.example.FocusChange.create({
        version: 1,
        state: State.BLURRED
    })
    worker.postMessage({
        contains: "focus_change",
        payload: proto.example.FocusChange.encode(msg).finish(),
    })
    console.log("sent", msg)
})
